# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.util import CLIError

from azure.mgmt.containerregistry.v2017_10_01.models import ReplicationUpdateParameters

from ._factory import get_acr_service_client
from ._utils import (
    get_resource_group_name_by_registry_name,
    validate_premium_registry
)


REPLICATIONS_NOT_SUPPORTED = 'Replications are only supported for managed registries in Premium SKU.'


def acr_replication_list(registry_name, resource_group_name=None):
    """Lists all the replications for the specified container registry.
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    """
    _, resource_group_name = validate_premium_registry(
        registry_name, resource_group_name, REPLICATIONS_NOT_SUPPORTED)
    client = get_acr_service_client().replications

    return client.list(resource_group_name, registry_name)


def acr_replication_create(location,
                           registry_name,
                           resource_group_name=None,
                           replication_name=None,
                           tags=None):
    """Creates a replication for a container registry.
    :param str location: The name of location
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    :param str replication_name: The name of replication
    """
    registry, resource_group_name = validate_premium_registry(
        registry_name, resource_group_name, REPLICATIONS_NOT_SUPPORTED)

    normalized_location = "".join(location.split()).lower()
    if registry.location == normalized_location:
        raise CLIError('Replication could not be created in the same location as the registry.')

    client = get_acr_service_client().replications

    return client.create(
        resource_group_name=resource_group_name,
        registry_name=registry_name,
        replication_name=replication_name or normalized_location,
        location=location,
        tags=tags
    )


def acr_replication_delete(replication_name,
                           registry_name,
                           resource_group_name=None):
    """Deletes a replication from a container registry.
    :param str replication_name: The name of replication
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    """
    _, resource_group_name = validate_premium_registry(
        registry_name, resource_group_name, REPLICATIONS_NOT_SUPPORTED)
    client = get_acr_service_client().replications

    return client.delete(resource_group_name, registry_name, replication_name)


def acr_replication_show(replication_name,
                         registry_name,
                         resource_group_name=None):
    """Gets the properties of the specified replication.
    :param str replication_name: The name of replication
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    """
    _, resource_group_name = validate_premium_registry(
        registry_name, resource_group_name, REPLICATIONS_NOT_SUPPORTED)
    client = get_acr_service_client().replications

    return client.get(resource_group_name, registry_name, replication_name)


def acr_replication_update_custom(instance, tags=None):
    if tags is not None:
        instance.tags = tags

    return instance


def acr_replication_update_get(client):  # pylint: disable=unused-argument
    """Returns an empty ReplicationUpdateParameters object.
    """
    return ReplicationUpdateParameters()


def acr_replication_update_set(client,
                               replication_name,
                               registry_name,
                               resource_group_name=None,
                               parameters=None):
    """Sets the properties of the specified replication.
    :param str replication_name: The name of replication
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    :param ReplicationUpdateParameters parameters: The replication update parameters object
    """
    resource_group_name = get_resource_group_name_by_registry_name(
        registry_name, resource_group_name)

    return client.update(
        resource_group_name=resource_group_name,
        registry_name=registry_name,
        replication_name=replication_name,
        tags=parameters.tags)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long

from azure.cli.core.commands import register_cli_argument
from azure.cli.core.commands.parameters import (
    resource_group_name_type,
    location_type,
    tags_type,
    deployment_name_type,
    get_resource_name_completion_list,
    quotes
)
from azure.cli.core.commands.validators import get_default_location_from_resource_group

from ._constants import (
    STORAGE_RESOURCE_TYPE,
    REGISTRY_RESOURCE_TYPE,
    WEBHOOK_RESOURCE_TYPE,
    REPLICATION_RESOURCE_TYPE,
    CLASSIC_REGISTRY_SKU,
    MANAGED_REGISTRY_SKU
)
from ._validators import validate_registry_name, validate_headers

register_cli_argument('acr', 'registry_name', options_list=('--name', '-n'), help='The name of the container registry. You can configure the default registry name using `az configure --defaults acr=<registry name>`', completer=get_resource_name_completion_list(REGISTRY_RESOURCE_TYPE), configured_default='acr')
register_cli_argument('acr', 'storage_account_name', help='Provide the name of an existing storage account if you\'re recreating a container registry over a previous registry created storage account. Only applicable to Classic SKU.', completer=get_resource_name_completion_list(STORAGE_RESOURCE_TYPE))
register_cli_argument('acr', 'sku', help='The SKU of the container registry', choices=MANAGED_REGISTRY_SKU + CLASSIC_REGISTRY_SKU)
register_cli_argument('acr', 'password_name', help='The name of password to regenerate', choices=['password', 'password2'])

register_cli_argument('acr', 'resource_group_name', resource_group_name_type)
register_cli_argument('acr', 'location', location_type)
register_cli_argument('acr', 'tags', tags_type)
register_cli_argument('acr create', 'admin_enabled', nargs='?', required=False, const='true', default='false', help='Indicates whether the admin user is enabled', choices=['true', 'false'])
register_cli_argument('acr update', 'admin_enabled', help='Indicates whether the admin user is enabled', choices=['true', 'false'])

register_cli_argument('acr', 'username', options_list=('--username', '-u'), help='The username used to log into a container registry')
register_cli_argument('acr', 'password', options_list=('--password', '-p'), help='The password used to log into a container registry')

register_cli_argument('acr repository delete', 'manifest', nargs='?', required=False, const='', default=None, help='The sha256 based digest of manifest to delete')
register_cli_argument('acr repository delete', 'yes', options_list=('--yes', '-y'), action='store_true', help='Do not prompt for confirmation')

register_cli_argument('acr create', 'registry_name', completer=None, validator=validate_registry_name)
register_cli_argument('acr create', 'deployment_name', deployment_name_type, validator=None)
register_cli_argument('acr create', 'location', location_type, validator=get_default_location_from_resource_group)
register_cli_argument('acr check-name', 'registry_name', completer=None)

register_cli_argument('acr webhook', 'registry_name', options_list=('--registry', '-r'), help='The name of the container registry. You can configure the default registry name using `az configure --defaults acr=<registry name>`', completer=get_resource_name_completion_list(REGISTRY_RESOURCE_TYPE), configured_default='acr')
register_cli_argument('acr webhook', 'webhook_name', options_list=('--name', '-n'), help='The name of the webhook', completer=get_resource_name_completion_list(WEBHOOK_RESOURCE_TYPE))
register_cli_argument('acr webhook', 'uri', help='The service URI for the webhook to post notifications.')
register_cli_argument('acr webhook', 'headers', nargs='+', help="Space separated custom headers in 'key[=value]' format that will be added to the webhook notifications. Use {} to clear existing headers.".format(quotes), validator=validate_headers)
register_cli_argument('acr webhook', 'actions', nargs='+', help='Space separated list of actions that trigger the webhook to post notifications.', choices=['push', 'delete'])
register_cli_argument('acr webhook', 'status', help='Indicates whether the webhook is enabled.', choices=['enabled', 'disabled'])
register_cli_argument('acr webhook', 'scope', help="The scope of repositories where the event can be triggered. For example, 'foo:*' means events for all tags under repository 'foo'. 'foo:bar' means events for 'foo:bar' only. 'foo' is equivalent to 'foo:latest'. Empty means events for all repositories.")
register_cli_argument('acr webhook create', 'webhook_name', completer=None)

register_cli_argument('acr replication', 'registry_name', options_list=('--registry', '-r'), help='The name of the container registry. You can configure the default registry name using `az configure --defaults acr=<registry name>`', completer=get_resource_name_completion_list(REGISTRY_RESOURCE_TYPE), configured_default='acr')
register_cli_argument('acr replication', 'replication_name', options_list=('--name', '-n'), help='The name of the replication.', completer=get_resource_name_completion_list(REPLICATION_RESOURCE_TYPE))
register_cli_argument('acr replication create', 'replication_name', help='The name of the replication. Default to the location name.', completer=None)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands import cli_command
from azure.cli.core.commands.arm import cli_generic_update_command
from azure.cli.core.util import empty_on_404
from azure.cli.core.profiles import supported_api_version, PROFILE_TYPE

from ._format import output_format
from ._factory import get_acr_service_client

if not supported_api_version(PROFILE_TYPE, max_api='2017-03-09-profile'):
    cli_command(__name__, 'acr credential show',
                'azure.cli.command_modules.acr.credential#acr_credential_show',
                table_transformer=output_format, exception_handler=empty_on_404)
    cli_command(__name__, 'acr credential renew',
                'azure.cli.command_modules.acr.credential#acr_credential_renew',
                table_transformer=output_format)

    cli_command(__name__, 'acr check-name', 'azure.cli.command_modules.acr.custom#acr_check_name')
    cli_command(__name__, 'acr list', 'azure.cli.command_modules.acr.custom#acr_list',
                table_transformer=output_format)
    cli_command(__name__, 'acr create', 'azure.cli.command_modules.acr.custom#acr_create',
                table_transformer=output_format)
    cli_command(__name__, 'acr delete', 'azure.cli.command_modules.acr.custom#acr_delete',
                table_transformer=output_format)
    cli_command(__name__, 'acr show', 'azure.cli.command_modules.acr.custom#acr_show',
                table_transformer=output_format, exception_handler=empty_on_404)
    cli_command(__name__, 'acr login', 'azure.cli.command_modules.acr.custom#acr_login')
    cli_command(__name__, 'acr show-usage', 'azure.cli.command_modules.acr.custom#acr_show_usage',
                table_transformer=output_format)
    cli_generic_update_command(
        __name__,
        'acr update',
        'azure.cli.command_modules.acr.custom#acr_update_get',
        'azure.cli.command_modules.acr.custom#acr_update_set',
        factory=lambda: get_acr_service_client().registries,
        custom_function_op='azure.cli.command_modules.acr.custom#acr_update_custom',
        table_transformer=output_format)

    cli_command(__name__, 'acr repository list',
                'azure.cli.command_modules.acr.repository#acr_repository_list')
    cli_command(__name__, 'acr repository show-tags',
                'azure.cli.command_modules.acr.repository#acr_repository_show_tags')
    cli_command(__name__, 'acr repository show-manifests',
                'azure.cli.command_modules.acr.repository#acr_repository_show_manifests')
    cli_command(__name__, 'acr repository delete',
                'azure.cli.command_modules.acr.repository#acr_repository_delete')

    cli_command(__name__, 'acr webhook list',
                'azure.cli.command_modules.acr.webhook#acr_webhook_list',
                table_transformer=output_format)
    cli_command(__name__, 'acr webhook create',
                'azure.cli.command_modules.acr.webhook#acr_webhook_create',
                table_transformer=output_format)
    cli_command(__name__, 'acr webhook delete',
                'azure.cli.command_modules.acr.webhook#acr_webhook_delete',
                table_transformer=output_format)
    cli_command(__name__, 'acr webhook show',
                'azure.cli.command_modules.acr.webhook#acr_webhook_show',
                table_transformer=output_format)
    cli_command(__name__, 'acr webhook get-config',
                'azure.cli.command_modules.acr.webhook#acr_webhook_get_config',
                table_transformer=output_format)
    cli_command(__name__, 'acr webhook list-events',
                'azure.cli.command_modules.acr.webhook#acr_webhook_list_events',
                table_transformer=output_format)
    cli_command(__name__, 'acr webhook ping',
                'azure.cli.command_modules.acr.webhook#acr_webhook_ping',
                table_transformer=output_format)
    cli_generic_update_command(
        __name__,
        'acr webhook update',
        'azure.cli.command_modules.acr.webhook#acr_webhook_update_get',
        'azure.cli.command_modules.acr.webhook#acr_webhook_update_set',
        factory=lambda: get_acr_service_client().webhooks,
        custom_function_op='azure.cli.command_modules.acr.webhook#acr_webhook_update_custom',
        table_transformer=output_format)

    cli_command(__name__, 'acr replication list',
                'azure.cli.command_modules.acr.replication#acr_replication_list',
                table_transformer=output_format)
    cli_command(__name__, 'acr replication create',
                'azure.cli.command_modules.acr.replication#acr_replication_create',
                table_transformer=output_format)
    cli_command(__name__, 'acr replication delete',
                'azure.cli.command_modules.acr.replication#acr_replication_delete',
                table_transformer=output_format)
    cli_command(__name__, 'acr replication show',
                'azure.cli.command_modules.acr.replication#acr_replication_show',
                table_transformer=output_format)
    cli_generic_update_command(
        __name__,
        'acr replication update',
        'azure.cli.command_modules.acr.replication#acr_replication_update_get',
        'azure.cli.command_modules.acr.replication#acr_replication_update_set',
        factory=lambda: get_acr_service_client().replications,
        custom_function_op='azure.cli.command_modules.acr.replication#acr_replication_update_custom',
        table_transformer=output_format)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.mgmt.containerregistry.v2017_10_01.models import (
    WebhookCreateParameters,
    WebhookUpdateParameters
)

from ._factory import get_acr_service_client
from ._utils import (
    get_resource_group_name_by_registry_name,
    validate_managed_registry
)


WEBHOOKS_NOT_SUPPORTED = 'Webhooks are only supported for managed registries.'


def acr_webhook_list(registry_name, resource_group_name=None):
    """Lists all the webhooks for the specified container registry."
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    """
    _, resource_group_name = validate_managed_registry(
        registry_name, resource_group_name, WEBHOOKS_NOT_SUPPORTED)
    client = get_acr_service_client().webhooks

    return client.list(resource_group_name, registry_name)


def acr_webhook_create(webhook_name,
                       uri,
                       actions,
                       registry_name,
                       location=None,
                       resource_group_name=None,
                       headers=None,
                       status='enabled',
                       scope=None,
                       tags=None):
    """Creates a webhook for a container registry.
    :param str webhook_name: The name of webhook
    :param str uri: The service URI for the webhook to post notifications
    :param str actions: The list of actions that trigger the webhook to post notifications
    :param str registry_name: The name of container registry
    :param str location: The name of location
    :param str resource_group_name: The name of resource group
    :param str headers: Custom headers that will be added to the webhook notifications
    :param str status: Indicates whether the webhook is enabled
    :param str scope: The scope of repositories where the event can be triggered
    """
    registry, resource_group_name = validate_managed_registry(
        registry_name, resource_group_name, WEBHOOKS_NOT_SUPPORTED)

    client = get_acr_service_client().webhooks

    return client.create(
        resource_group_name,
        registry_name,
        webhook_name,
        WebhookCreateParameters(
            location=location or registry.location,
            service_uri=uri,
            actions=actions,
            custom_headers=headers,
            status=status,
            scope=scope,
            tags=tags
        )
    )


def acr_webhook_delete(webhook_name,
                       registry_name,
                       resource_group_name=None):
    """Deletes a webhook from a container registry.
    :param str webhook_name: The name of webhook
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    """
    _, resource_group_name = validate_managed_registry(
        registry_name, resource_group_name, WEBHOOKS_NOT_SUPPORTED)
    client = get_acr_service_client().webhooks

    return client.delete(resource_group_name, registry_name, webhook_name)


def acr_webhook_show(webhook_name,
                     registry_name,
                     resource_group_name=None):
    """Gets the properties of the specified webhook.
    :param str webhook_name: The name of webhook
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    """
    _, resource_group_name = validate_managed_registry(
        registry_name, resource_group_name, WEBHOOKS_NOT_SUPPORTED)
    client = get_acr_service_client().webhooks

    return client.get(resource_group_name, registry_name, webhook_name)


def acr_webhook_update_custom(instance,
                              uri=None,
                              actions=None,
                              headers=None,
                              status=None,
                              scope=None,
                              tags=None):
    if uri is not None:
        instance.service_uri = uri

    if actions is not None:
        instance.actions = actions

    if headers is not None:
        instance.custom_headers = headers

    if status is not None:
        instance.status = status

    if scope is not None:
        instance.scope = scope

    if tags is not None:
        instance.tags = tags

    return instance


def acr_webhook_update_get(client):  # pylint: disable=unused-argument
    """Returns an empty WebhookUpdateParameters object.
    """
    return WebhookUpdateParameters()


def acr_webhook_update_set(client,
                           webhook_name,
                           registry_name,
                           resource_group_name=None,
                           parameters=None):
    """Sets the properties of the specified webhook.
    :param str webhook_name: The name of webhook
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    :param WebhookUpdateParameters parameters: The webhook update parameters object
    """
    resource_group_name = get_resource_group_name_by_registry_name(
        registry_name, resource_group_name)

    return client.update(resource_group_name, registry_name, webhook_name, parameters)


def acr_webhook_get_config(webhook_name,
                           registry_name,
                           resource_group_name=None):
    """Gets the configuration of service URI and custom headers for the webhook.
    :param str webhook_name: The name of webhook
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    """
    _, resource_group_name = validate_managed_registry(
        registry_name, resource_group_name, WEBHOOKS_NOT_SUPPORTED)
    client = get_acr_service_client().webhooks

    return client.get_callback_config(resource_group_name, registry_name, webhook_name)


def acr_webhook_list_events(webhook_name,
                            registry_name,
                            resource_group_name=None):
    """Lists recent events for the specified webhook.
    :param str webhook_name: The name of webhook
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    """
    _, resource_group_name = validate_managed_registry(
        registry_name, resource_group_name, WEBHOOKS_NOT_SUPPORTED)
    client = get_acr_service_client().webhooks

    return client.list_events(resource_group_name, registry_name, webhook_name)


def acr_webhook_ping(webhook_name,
                     registry_name,
                     resource_group_name=None):
    """Triggers a ping event to be sent to the webhook.
    :param str webhook_name: The name of webhook
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    """
    _, resource_group_name = validate_managed_registry(
        registry_name, resource_group_name, WEBHOOKS_NOT_SUPPORTED)
    client = get_acr_service_client().webhooks

    return client.ping(resource_group_name, registry_name, webhook_name)
